@@ -23,6 +23,14 @@ capture_output <- function(f, ...) {
    expect_false(is.null(plate))
  })
 
+ test_that("test pipeline classes", {
+   lipsum_214 <- system.file("extdata", "dat_1.dat", package = "normfluodbf")
+   plate = setup_plate(init_plate(child_type = '96well_plate'))
+   plate <- plate %>%
+     upload_data(file = lipsum_214, tnp = 3, cycles = 40, rows_used = c('A','B','C'), norm_scale = 'raw')
+   expect_equal("96well_plate" , class(plate)[1])
+ })
+
  test_that("test pipeline with run steps", {
    lipsum_214 <- system.file("extdata", "dat_1.dat", package = "normfluodbf")
    plate = setup_plate(init_plate())
